#include "lokdecoder.h"
.text
//************************************************************************************************
//* DCC Lokdekoder mit ATtiny15																	*
//*																								*
//*																								*
//*	RESET		  PB5 = 1	8 = VCC 														*
//*	Licht Rckwrts   PB4 = 2	7 = PB2 DCC Eingang												*
//*	Licht Vorwrts    PB3 = 3	6 = PB1 Ausgang Motor PWM										*
//*       		  GND = 4	5 = PB0 Ausgang Vorwrts/Rckwrts 								*
//*																								*
//************************************************************************************************

SIG_INTERRUPT0:	
	in intsreg,_SFR_IO_ADDR(SREG)		//_SFR_IO_ADDR(SREG) sichern
	clr inttemp				//Timer anhalten
	out _SFR_IO_ADDR(TCCR0B),inttemp
	ldi inttemp,177				//Zähler setzen ((255-161)+5 die beim timerneustart schon verganngen sind)
	out _SFR_IO_ADDR(TCNT0),inttemp
	in inttemp,_SFR_IO_ADDR(TIFR0)		//Überlauf timer0 lschen
	cbr inttemp,0b00000010
	out _SFR_IO_ADDR(TIFR0),inttemp
	sbr flag,(1<<MESSEN)			//für bit messen ein
	cbr flag,(1<<KEINDCC)			//bit für keine befehle da löschen
	ldi inttemp,0b00000001			//timer mit Teiler auf ck starten
	out _SFR_IO_ADDR(TCCR0B),inttemp
	out _SFR_IO_ADDR(SREG),intsreg		//_SFR_IO_ADDR(SREG) zurück
	reti					//und fertig

//************************************
//*		Timer_0_interrupt			*
//************************************
// wird ca 78us nach ext_int ausgelst und liest dann bit von int0/b2 ein
// danach max 10000us auf neuen ext_int warten
// dazu teiler auf 64 und zähler auf 5

SIG_OVERFLOW0:		
	in intsreg,_SFR_IO_ADDR(SREG)		// sreg sichern
	clr inttemp				//timer anhalten
	out _SFR_IO_ADDR(TCCR0B),inttemp
	in inttemp,_SFR_IO_ADDR(PINB)		//port einlesen
	sbrs flag,MESSEN			//wenn messen aktiv 1x springen
	rjmp warten_aktiv			//sonst gehe zu
	com inttemp				//port umdrehen
	bst inttemp,1				//bit B1 holen
	bld flag,BIT				//und in flag bit0 sichern
	sbr flag,(1<<NEUESBIT)			//neues bit da setzen
	cbr flag,(1<<MESSEN)			//flag für messen aus
	ldi inttemp,TVAL			//Zähler auf 5
	out _SFR_IO_ADDR(TCNT0),inttemp
	ldi inttemp,0b00000011			//timer mit Teiler auf ck/64 starten
	out _SFR_IO_ADDR(TCCR0B),inttemp
	out _SFR_IO_ADDR(SREG),intsreg
	reti
warten_aktiv:					//ca 10000us kein bit mehr eingetroffen
	sbr flag,(1<<KEINDCC)			//bit für keine befehle setzen
	out _SFR_IO_ADDR(SREG),intsreg
	reti

//************************************
//*		EEprom byte lesen			*
//************************************
//liest ein Byte aus Addresse eead in temp ein zum CVregister lesen

eepromlesen:	
	sbic _SFR_IO_ADDR(EECR),EEPE			// wird noch geschrieben ? EECR Bit1(EEWE)=1
        rjmp eepromlesen  		    	// dann Schleife
	out _SFR_IO_ADDR(EEARL),eead			// eeprom addresse low setzen bei kleinen eeprom eear bei groem eearl	
	sbi _SFR_IO_ADDR(EECR),EERE 			// Setze EEProm-Lesebefehl EECR Bit0(EERE)=1
	in temp,_SFR_IO_ADDR(EEDR)			// daten einlesen
	ret

//************************************
//*		EEprom byte schreiben		*
//************************************
//schreibt ein Byte aus temp in Addresse eead zum CVregister schreiben

eepromschreiben:
	sbic _SFR_IO_ADDR(EECR),EEPE		// wird noch geschrieben ? EECR Bit1(EEWE)=1
	rjmp eepromschreiben  		    	// dann Schleife
	out _SFR_IO_ADDR(EEARL),eead		// bei kleinen eeprom eear bei groem eearl
	out _SFR_IO_ADDR(EEDR),temp		// und daten setzen
        cli       			     	// alle Interrupts ausschalten
	sbi _SFR_IO_ADDR(EECR),EEMPE 		// Setze EEProm Master-Schreibbit 
        sbi _SFR_IO_ADDR(EECR),EEPE		// Setze EEProm Schreib-Befehl
	sei  			          	// alle Interrupts wieder einschalten
	ret					// Rcksprung

//************************************
//*		Funktionen setzen			*
//************************************
//wird zyklisch aus dem Hauptprogramm aufgerufen und setzt:
//1.MotorPWM ber Timer1 und MotorRichtung an PB2
//2.erzeugt Licht PWM genauigkeit ist nicht sehr hoch, hängt vom Hauptprogramm ab, reicht für licht aus
// und setzt die Ausgänge PB3 und PB4
//3.setzt die Funktionsausgänge wenn Portpins vorhanden

fktset:
	in fktsreg,_SFR_IO_ADDR(SREG)		// sreg sichern
	sbrc dccreg,FL				// wenn Licht aus 1x springen
	rjmp lichtan				// wenn an gehe zu...
	cbi _SFR_IO_ADDR(PORTB),LICHTV		// lösche lichtausgänge
	cbi _SFR_IO_ADDR(PORTB),LICHTR
	rjmp faset				// gleich weiter mit...
lichtan:
//	inc pwmzaehler				// Zlher erhhen
//	cpi pwmzaehler,100			// wenn 100 dann wieder auf 0 setzen
//	brne .+2
//	clr pwmzaehler
//	cbi _SFR_IO_ADDR(PORTB),LICHTR  	// Licht Rckwrts aus
//	cbi _SFR_IO_ADDR(PORTB),LICHTV		// Licht Vorwrts aus
//	cp pwmzaehler,flpwm			// Vergleich mit PWM Wert
//	brlo faset				// wenn Zaehler kleiner berspringen					
//	sbrs dccreg,RICHTUNG			// 1x springen bei vorwrts
//	rjmp lichtrueckwaerts
lichtvorwaerts:
	sbi _SFR_IO_ADDR(PORTB),LICHTV		// Licht Vorwrts an
//	rjmp faset				// weiter mit...
lichtrueckwaerts:
	sbi _SFR_IO_ADDR(PORTB),LICHTR		// Licht Rckwrts an

faset:						// Funktionsausgnge setzen
	sbrc dccreg,F1				// wenn F1 gesetz:
	sbi _SFR_IO_ADDR(PORTB), PINF1		// PB4 einschalten
	sbrs dccreg,F1				// wenn F1 aus
	cbi _SFR_IO_ADDR(PORTB), PINF1		// PB4 ausschalten
/*
	sbrc dccreg,F2				// wenn F2 gesetz:
	sbi _SFR_IO_ADDR(PORTB), PINF2		// PB5 einschalten
	sbrs dccreg,F2				// wenn F2 aus
	cbi _SFR_IO_ADDR(PORTB), PINF2		// PB5 ausschalten
	sbrc dccreg,F3				// wenn F3 gesetz:
	sbi _SFR_IO_ADDR(PORTB), PINF3		// PB6 einschalten
	sbrs dccreg,F3				// wenn F3 aus
	cbi _SFR_IO_ADDR(PORTB), PINF3		// PB6 ausschalten
	sbrc dccreg,F4				// wenn F4 gesetz:
	sbi _SFR_IO_ADDR(PORTB), PINF4		// PB7 einschalten
	sbrs dccreg,F4				// wenn F4 aus
	cbi _SFR_IO_ADDR(PORTB), PINF4		// PB7 ausschalten
*/
	ret					// Rücksprung

//************************************************************************************************
//*					Hauptprogramm																*
//************************************************************************************************

main:						// Programm Kaltstart
  	cli            			// alle Interrupts ausschalten

	ldi temp,0b00011101		// PB0/2/3/4 als Ausgang, PB1/5 als Eingang
  	out _SFR_IO_ADDR(DDRB),temp
	clr null			// alles auf low
	out _SFR_IO_ADDR(PORTB),null

  	ldi temp,0b01000011		// Bit0/1 Interupt Sense Control
  	out _SFR_IO_ADDR(MCUCR),temp	// 0 0 Low Level of INT0 generate Interrupt
						// 0 1 jeder Wechsel of INT0
						// 1 0 die fallende Flanke of INT0
						// 1 1 die steigende Flanke von INT0 erzeugt Interrupt
						// Bit3/4 verschiedene Sleep Modes
						// Bit5 Sleep Enable
						// Bit6 global Pull-up PortB aus
	  
  	ldi temp,0b01000000		// Bit5 PIN change Interrupt enable
  	out _SFR_IO_ADDR(GIMSK),temp	// Bit6 INT0 Interrupt enable

  	ldi temp,0b00000010		// Bit1 Timer/Counter0 Overflow Interrupt Enable
  	out _SFR_IO_ADDR(TIMSK0),temp	// Bit2 Timer/Counter1 Overflow Interrupt Enable 
						// Bit6 Timer/Counter1 Output Compare Interrupt Enable	  
//	ldi eead,EEPWMPERI		// PWM Teiler fr Motor laden
//	rcall	eepromlesen
//	mov pwmaus,temp			// in PWM_aus Variable kopieren
//	ldi temp,5			// PWM Teiler berechnen und ablegen
//	add pwmaus,temp			// (siehe oben teiler 0101 = CK = 6452Hz usw)
//	sbr pwmaus,0b01000000		// 
//	mov pwman,pwmaus		// PWM_an Variable setzen
//	sbr pwman,0b00100000
//	out TCCR1,pwmaus		// setze PWM Teiler und trenne Ausgang ab
//	out TCNT1,null			// lsche Timer 1
  	            
  	ldi temp,0b01000000		// lsche externen Interupt 0
  	out _SFR_IO_ADDR(GIFR),temp
  
  	ldi temp,0b00000010		// lösche Timer Interupt
  	out _SFR_IO_ADDR(TIFR0),temp 	  

  	ldi eead,EEADRESSE		// hole Adresse
  	rcall eepromlesen
  	mov adresse,temp

  	ldi eead,EESTART		// hole Startspannung
  	rcall eepromlesen
  	mov vstart,temp
	  
  	ldi eead,EEINCWERT		// hole Beschleunigung
  	rcall eepromlesen
  	mov incwert,temp
	inc incwert			// 0=aus auf 1 setzten( kleinster wert ist 1 im Programmablauf)
  
  	ldi eead,EEDECWERT		// hole Verzögerung
  	rcall eepromlesen
  	mov decwert,temp
	inc decwert			// wie incwert
	  
  	ldi eead,EEVMAX			// hole Höchstgeschwindigkeit
  	rcall eepromlesen
  	mov vmax,temp

	ldi eead,EEFLPWM		// hole Licht PWM Wert
  	rcall eepromlesen
  	mov flpwm,temp
	  
getcv: ldi eead,EECV29			// Konfig bits holen 
 	rcall eepromlesen
  	bst temp,1			// 14/28 Fahrstufen
  	bld dccreg,FAHRSTUFEN
  	bst temp,0			// Reversierbetrieb
  	bld dccreg,RICHTUNGINV
	bst temp,6			// CV29 bit6 holen und kurz in T bit merken
	clr flag			// flag setzen erst lschen
	sbr flag,(1<<RESETBIT)		// dann setze resetbit
	bld flag,PROGSW1		// dann fge T bit ein
	bst temp,5				// CV29 bit5 holen und
	bld flag,PROGSW2		// und ablegen
	bst temp,4				// CV29 bit4 holen
	bld flag,VMERKEN		// und ablegen

	cbr dccreg,(1<<FL)		// Licht aus damit nicht aufflackert bei stromunterbrechung
  	clr temp			// bereinige alle bentigten Register
	clr vsoll
	clr vist
  
  	sei 				// aktiviere alle gesetzten Interupts

/*
	sbrs flag,VMERKEN		// wenn V merken an 1x springen
	rjmp hauptschleife		// sonst gehe gleich zu..
	ldi eead,EEADVIST		// EEPROM Zelle fr EEV_ist holen
	rcall eepromlesen
	mov EEV_ist,temp
	mov eead,EEVIST			// letzte Vsoll holen
	rcall eepromlesen
	bst temp,0			// richtung holen (wurde in bit0 berschrieben)
	bld dccreg,RICHTUNG		// und ablegen
	cbr temp,0b00000001		// danach aus v_ist wieder entfernen
	mov V_ist,temp			// und V_ist setzen
pwmstartset: 
	tst v_ist				// ist Speed 0
  	brne pwmstartout			// dann 0 ansonsten Out
								// Timer/Counter1 PWM vom 
	rjmp hauptschleife		// und weiter zur..
pwmstartout:
  	out _SFR_IO_ADDR(OCR1A),vist	// Wert fr Speed nach Timer 1 Compare
  	out _SFR_IO_ADDR(TCCR1),pwman	// Timer/Counter1 PWM Ausgang ein
	sbrs dccreg,RICHTUNG		// wenn Fahrtrichtung vorwrts 1x springen
  	rjmp revstartout		// gehe zu Ausgang auf rckwrts setzen
  	sbi _SFR_IO_ADDR(PORTB),PB0	// setze Ausgang auf vorwrts
	rjmp hauptschleife		// und weiter zur..
revstartout:
	cbi  _SFR_IO_ADDR(PORTB),PB0	// setze Ausgang auf rckwrts
*/
hauptschleife:
  	clr bitzaehler     		// loesche den BitCounter wieder
        clr bytea			// loesche die Empfangsregister wieder
        clr byteb
        clr bytec
        clr byted
        clr bytee
        clr bytef
	cbr flag,(1<<NEUESBIT | 1<<BIT | 1<<KEINDCC)
  	
preamble: 				// Prüfe Präambel nach NMRA-DCC
	rcall fktset			// hier in Wartezeit Funktionen setzen
  	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
  	rjmp .-4			// Schleife
  	cbr flag,(1<<NEUESBIT)		// lsche flag fr neues Bit
  	sbrs flag,BIT			// wenn das neue Bit = 1 dann erhhe Zhler
  	rjmp preend			// ansonsten Signal fr Prambel Ende
  	inc bitzaehler			//
  	rjmp preamble			// Schleife

preend: cpi bitzaehler,10		// wenn Bitzhler kleiner 10
	brlo prefehl			// dann Fehler		  
	clr bitzaehler			// lsche Zhler
	rjmp lesebytea  		// hole Adresse (kurze NMRA-Adresse )
 
prefehl:
//	sbi _SFR_IO_ADDR(PORTB), PINF1
	rjmp 	hauptschleife		// Endlosschleife

lesebytea: 				// hole erstes von bis zu sechs Bytes
	rcall fktset			// hier in Wartezeit Funktionen setzen
	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
	rjmp .-4				// Schleife
	cbr flag,(1<<NEUESBIT)		// lösche flag für neues Bit
	cpi bitzaehler,8		// vergleiche mit Bytelnge		//
	breq byteaende			// und springe wenn Byte eingelesen
	sec							// setzte Carry
	sbrs flag,BIT			// wenn empfangenes Bit=1 1x springen
	clc							// lsche Carry
	rol bytea				// schiebe Carry von rechts in das Register
	inc bitzaehler			// erhhe Zhler
	rjmp lesebytea		  	// Schleife
byteaende:
	mov temp,bytea		// sichere Byte
  	sbrc flag,BIT			// springe 1x wenn empfangenes Bit=0 
  	rjmp hauptschleife		// sonst Byte nicht korrekt abgeschlossen
  	clr bitzaehler			// lsche Zhler

lesebyteb:				// hole zweites von bis zu sechs Bytes
	rcall fktset			// hier in Wartezeit Funktionen setzen
  	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
  	rjmp .-4				// Schleife
  	cbr flag,(1<<NEUESBIT)	// lsche flag fr neues Bit
  	cpi bitzaehler,8		// vergleiche mit Bytelnge		//
  	breq bytebende			// und springe wenn Byte eingelesen
  	sec							// setzte Carry
  	sbrs flag,BIT			// wenn empfangenes Bit=1 1x springen
  	clc							// lsche Carry
  	rol byteb				// schiebe Carry von rechts in das Register
  	inc bitzaehler			// erhhe Zhler
  	rjmp lesebyteb		  	// Schleife
bytebende:
	eor temp,byteb			// bilde XOR	
  	sbrc flag,BIT			// springe 1x wenn empfangenes Bit=0 
  	rjmp hauptschleife		// sonst Byte nicht korrekt abgeschlossen
  	clr bitzaehler			// lsche Zhler

lesebytec: 				// hole erstes von bis zu sechs Bytes
	rcall fktset			// hier in Wartezeit Funktionen setzen
	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
	rjmp .-4				// Schleife
	cbr flag,(1<<NEUESBIT)		// lösche flag für neues Bit
	cpi bitzaehler,8		// vergleiche mit Bytelnge		//
	breq bytecende			// und springe wenn Byte eingelesen
	sec							// setzte Carry
	sbrs flag,BIT			// wenn empfangenes Bit=1 1x springen
	clc							// lsche Carry
	rol bytec				// schiebe Carry von rechts in das Register
	inc bitzaehler			// erhhe Zhler
	rjmp lesebytec		  	// Schleife
bytecende:
	eor temp, bytec
  	sbrc flag,BIT			// springe 1x wenn empfangenes Bit=0 
  	rjmp datenpruefen		// sonst Byte nicht korrekt abgeschlossen
  	clr bitzaehler			// lsche Zhler

lesebyted: 				// hole erstes von bis zu sechs Bytes
	rcall fktset			// hier in Wartezeit Funktionen setzen
	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
	rjmp .-4				// Schleife
	cbr flag,(1<<NEUESBIT)		// lösche flag für neues Bit
	cpi bitzaehler,8		// vergleiche mit Bytelnge		//
	breq bytedende			// und springe wenn Byte eingelesen
	sec							// setzte Carry
	sbrs flag,BIT			// wenn empfangenes Bit=1 1x springen
	clc							// lsche Carry
	rol byted				// schiebe Carry von rechts in das Register
	inc bitzaehler			// erhhe Zhler
	rjmp lesebyted		  	// Schleife
bytedende:
	eor temp, byted
  	sbrc flag,BIT			// springe 1x wenn empfangenes Bit=0 
  	rjmp datenpruefen		// sonst Byte nicht korrekt abgeschlossen
  	clr bitzaehler			// lsche Zhler

lesebytee: 				// hole erstes von bis zu sechs Bytes
	rcall fktset			// hier in Wartezeit Funktionen setzen
	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
	rjmp .-4				// Schleife
	cbr flag,(1<<NEUESBIT)		// lösche flag für neues Bit
	cpi bitzaehler,8		// vergleiche mit Bytelnge		//
	breq byteeende			// und springe wenn Byte eingelesen
	sec							// setzte Carry
	sbrs flag,BIT			// wenn empfangenes Bit=1 1x springen
	clc							// lsche Carry
	rol bytee				// schiebe Carry von rechts in das Register
	inc bitzaehler			// erhhe Zhler
	rjmp lesebytee		  	// Schleife
byteeende:
	eor temp, bytee
  	sbrc flag,BIT			// springe 1x wenn empfangenes Bit=0 
  	rjmp datenpruefen		// sonst Byte nicht korrekt abgeschlossen
  	clr bitzaehler			// lsche Zhler

lesebytef: 				// hole erstes von bis zu sechs Bytes
	rcall fktset			// hier in Wartezeit Funktionen setzen
	sbrs flag,NEUESBIT		// wenn kein neues Bit dann
	rjmp .-4				// Schleife
	cbr flag,(1<<NEUESBIT)		// lösche flag für neues Bit
	cpi bitzaehler,8		// vergleiche mit Bytelnge		//
	breq bytefende			// und springe wenn Byte eingelesen
	sec							// setzte Carry
	sbrs flag,BIT			// wenn empfangenes Bit=1 1x springen
	clc							// lsche Carry
	rol bytef				// schiebe Carry von rechts in das Register
	inc bitzaehler			// erhhe Zhler
	rjmp lesebytef		  	// Schleife
bytefende:
	eor temp, bytef
  	sbrc flag,BIT			// springe 1x wenn empfangenes Bit=0 
  	rjmp hauptschleife		// sonst Byte nicht korrekt abgeschlossen


datenpruefen:				// Datenprfung und bergabe
	rcall fktset			// hier in Wartezeit Funktionen setzen
	tst temp			// prfe temp auf 0			 	 
  	brne prueffehl			// wenn nicht 0 dann Fehler

  	sbrc flag,0			// wenn 1 war reset da gehe zu
	rjmp progmode			// Programmiermodus CV-Lesen, Schreiben, Bitmanipulation
					// sonst normal weiter	  
  	ldi temp,0 			// prüfe Adresse
  	cp bytea,temp			// wenn 0 dann 
  	breq analle			// Broadcast an alle
	  
  	ldi temp,255			// prüfe Adresse
  	cp bytea,temp			// wenn 255 dann
  	breq idle			// IDLE an alle
	  
  	cp adresse,bytea		// wenn eigene Adresse
  	breq pruefenende		// dann test OK
	  	  
idle:					// alle Decoder sind IDLE (machen nichts bei diesem befehl)
	  	  
prueffehl:
  	rjmp hauptschleife		// Endlosschleife

analle: 				// alle Decoder sind angesprochen
  	  
pruefenende:				// Adresse und Prfsumme OK
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen  
 					// Funktionen auswhlen
  	mov temp,byteb			// Daten sichern
  	andi temp,0b11100000 		// Datenbits ausblenden
	cpi temp,0b00000000		// Vergleich auf dcc reset
	brne n_reset			// dec reset bearbeiten
	rjmp main			// Decoder reset
//	cpi 	temp,0b00100000		// Vergleiche auf erweiterte Operation (enthlt befehl fr 128 Fahrstufen)
//	breq 	erweit_Befehle		// wenn ja gehe zu
n_reset:cpi temp,0b01000000		// Vergleiche mit V Rckwrts
  	breq vcalc  			// wenn ja gehe zu
  	cpi temp,0b01100000		// Vergleiche mit V Vorwrts
  	breq vcalc
  	cpi temp,0b10000000		// Vergleiche mit Funktionsgruppe F1-F4
  	breq fgruppe1
					// alles andere wird nicht unterstzt -> verwerfen und zurck
  	rjmp hauptschleife		// Endlosschleife


vcalc:					// V Berechnung 14 und 28 Fahrstufen
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen
					// zuerst Richtung schreiben
  	mov temp,byteb			// Daten sichern
  	tst vist			// ist V ungleich 0 ?
  	brne vset			// wenn ja gehe zu Fahrstufen setzen
  	sbrc dccreg,RICHTUNGINV		// wenn Fahrrichtung nicht reversiert 1x springen
        com byteb			// ansonsten wird Byte gedreht	
        bst byteb,5			// hole Richtungsbit
  	bld dccreg,RICHTUNG		// nach DCC Register
vset:					// Fahrstufen setzen
	sbrc dccreg,FAHRSTUFEN		// 0=14 Fahrstufen 1x springen
	rjmp v28set			// wenn 28 gehe zu
	bst temp,4			// Lichtbit holen
	bld dccreg,FL			// und in FL ablegen
	cbr temp,0b11110000		// bits fr kennung, richtung und licht lschen

v28set:					// ab hier fr 14 und 28 Fahrstufen gleich
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen
	bst temp,4			// bit0 fr 28 Fahrstufen sichern
	cbr temp,0b11110000		// bits fr kennung und richtung lschen
	cpi temp,0b00000001		// auf notstop testen
	breq notstop			// wenn gleich gehe zu
	rol temp			// 1x nachlinks verschieben
	bld temp,0			// und bit0 wieder einfgen
	mov byteb,temp			// V_max begrenzung auf 50/62,5%/75/88,5%/100%
	add temp,byteb			// V auf 37,5%
	add temp,byteb
	cpi vmax,4
	breq v50set
	cpi vmax,3
	breq v625set
	cpi vmax,2
	breq v75set
	cpi vmax,1
	breq v885set
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen
v100set:add temp,byteb
v885set:add temp,byteb		
v75set:	add temp,byteb
v625set:add temp,byteb	
v50set:	add temp,byteb
vstartset:
	cpi temp,0			// bei stop berspringen
	breq vsollset
	cp temp,vstart			// Startgeschw setzen 
	brsh vsollset			// wenn neue V kleiner V_start
	mov temp,vstart
vsollset:
	cp vsoll,temp			// neuen V wert empfangen?
	brne .+2			// wenn ja 1x springen
	rjmp hauptschleife		// wenn kein neuer wert gehe zurck
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen
	mov vsoll,temp			// neue soll geschw. ablegen
	sbrs flag,VMERKEN		// wenn V merken an 1x springen
	rjmp hauptschleife		// sonst gehe gleich zu..
	ldi pwmzaehler,99		// vor EEprom schreiben Licht bei PWM Betrieb aus (blitzt sonst auf)
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen
	bst dccreg,RICHTUNG		// Richtungsbit holen
	bld temp,0			// und bit0 von Vsoll in Temp überschreiben
	mov eead,eevist			// V wert im EEprom merken
	rcall eepromschreiben
  	rjmp hauptschleife		// Endlosschleife

notstop:
  	clr vist			// Notstop machen
  	clr vist			//
					// Timer/Counter1 PWM vom 
//        out  	TCCR1,PWM_aus		// Ausgang trennen
//        cbi  	PortB,0			// alles auf Null
  	rjmp hauptschleife		// Endlosschleife		

fgruppe1:				// Befehle fr FG1 einlesen
	rcall fktset			// jetzt ersteinmal wieder Funktionen setzen
	bst byteb,F1			// F1 bis F4 sichern
	bld dccreg,F1
	bst byteb,F2
	bld dccreg,F2
	bst byteb,F3
	bld dccreg,F3
	bst byteb,F4
	bld dccreg,F4
	sbrs dccreg,FAHRSTUFEN		// 1=28 Fahrstufen 1x springen
	rjmp hauptschleife		// bei 14 fertig Endlosschleife
	bst byteb,FL			// bei 28 noch Lichtbit sichern
	bld dccreg,FL
	rjmp hauptschleife		// und dann zurck				

progmode:
  	mov temp,bytea			// Daten sichern
	cpi temp,0b00000000		// Vergleich auf dcc reset
	breq noch_dcc_reset		// dec reset bearbeiten
	cpi temp,0b01111100		// prfe Funktionscode auf CV Write Byte Schreiben
	breq progbyte			// 	
	cbr flag,(1<<RESETBIT)		// wenn etwas anderes kommt lsche Reset und gehe zu
	rjmp hauptschleife

noch_dcc_reset:
	rjmp main			// gehe nocheimal zu

progbyte:
	cp progtest,byted   		// vergleiche mit vorherigen Daten (XOR-Byte)
  	brne progwrack			// springe wenn ungleich
	mov temp,byteb			// teste ob cv 29 geschrieben werden soll?
	cpi temp,29
	brsh progwrack			// CVreg größer 29 wird nicht unterstzt -> abbrechen
	sbrs flag,PROGSW1		// wenn 0
	rjmp progsw2_test		// gehe zu ...
	ldi eead,EECV29			// progschalter wieder auf normal schalten
	rcall eepromlesen		// es wird genau 1x schreiben in FL PWM erlaubt
	cbr temp,0b01000000		// 
	rcall eepromschreiben		//
	ldi temp,(EEFLPWM-1)		// sonst setzte auf FL PWM Byte
	mov byteb,temp
	rjmp prognormal
progsw2_test:
	sbrs flag,PROGSW2		// wenn 0
	rjmp prognormal			// gehe zu ...
	ldi eead,EECV29			// progschalter wieder auf normal schalten
	rcall eepromlesen		// es wird genau 1x schreiben in Motor PWM erlaubt
	cbr temp,0b00100000		// 
	rcall eepromschreiben		//
	ldi temp,(EEPWMPERI-1)		// sonst setzte auf FL PWM Byte
	mov byteb,temp
prognormal:
  	mov eead,byteb			// setzte Schreib-Adresse
	inc eead			// weil im eeprom zelle0 unbenutzt bleibt (alles 1x nach obenverschoben)
  	mov temp,bytec			// hole Daten
  	rcall eepromschreiben		// und schreibe ins EEPROM

//  	sbi _SFR_IO_ADDR(PORTB), PB1	// setze einen Motorausgang
	  
progbestaetigung:			// Acknowledge
//  	cli        			// alle Interrupts ausschalten
// 	ldi temp,63   			// lade den Timer/Counter1
//	out _SFR_IO_ADDR(TCNT1),temp	// setze Zeit fr Acknowlege
// 	ldi  	temp,0b00001100		// 1 1 0 0 CK/128 fr Resetzeit und ACK
//        out  	TCCR1,temp		// 
//        in   	temp,TIFR		// hole Timer Interuptflag
//  	sbrs 	temp,TOV1		// wenn Ackzeit abgelaufen
//  	rjmp 	PC-2			// Schleife
//        ldi  	temp,0b00000110		// lsche Timer Interupts
//  	out  	TIFR,temp
//        out 	PortB,null  		// lsche Motorausgang
//	cbr	flag,(1<<resetbit)	// lsche Reset und dann normal weiter
//        sei            			// alle Interrupts wieder einschalten

progwrack: 
	mov progtest,byted		// Datencheck		  				
        rjmp hauptschleife		// Endlosschleife
		
