//************************************************************************************************
// DCC Lokdekoder mit ATtiny13									*
//																								*
//																								*
//	RESET		  PB5 = 1	8 = VCC 														*
//	Licht Rckwrts   PB4 = 2	7 = PB2 DCC Eingang												*
//	Licht Vorwrts    PB3 = 3	6 = PB1 Ausgang Motor PWM										*
//       		  GND = 4	5 = PB0 Ausgang Vorwrts/Rckwrts 								*
//																								*
//************************************************************************************************
#ifndef _LOKDEC_H

#define _LOKDEC_H

#include <avr/io.h>

// registers

//;********************************
//;* FUSE info:					*
//;*								*
//;* BODLEVEL:	0/0 (4.3V)				*
//;* RSTDISBL:	nein			*
// -> fuse high byte: 0xf9
//;* SPIEN:		ja				*
// eesave: 1
// wdton: 1
// ckdiv: 0
// sut1: 1
// sut0: 0
// CKSEL1: 1
// CKSEL0: 0
// -> fuse low byte: 0x6a
//;********************************
//;* Konstantendef.				*
//;********************************

#define	EEOSCCAL 63	// Speicherplatz OSCCAL im EEPROM
#define	EEADRESSE 1	// Speicherplatz eigene Adresse im EEPROM
#define EESTART	2	// Speicherplatz Start-Speed im EEPROM
#define EEINCWERT 3	// Speicherplatz Beschleunigungskonstante im EEPROM
#define EEDECWERT 4	// Speicherplatz Verzgerungskonstante im EEPROM
#define EEVMAX 5	// Speicherplatz Hchstgeschwindigkeit im EEPROM
#define	EEFLPWM 6	// Speicherplatz Licht PWM im EEPROM
#define	EEADVIST 7	// Speicherplatz EEPROM Adresse mit V_ist im EEPROM bit0 mit Richtung berschrieben!
#define	EEPWMPERI 9	// Speicherplatz Motor PWM Periode
#define	EECV29 	29	// Speicherplatz Konfigurationsregister

//Flagregister
#define	RESETBIT 0	// ist nach dec reset gesetzt
#define	PROGSW1 1	// progschalter 1 zum prog. mit roco lokmaus
#define	PROGSW2 2	// progschalter 2
#define	VMERKEN 3
#define	BIT 4
#define	NEUESBIT 5
#define	MESSEN 6
#define	KEINDCC 7	// fr analogbetrieb(noch nicht aktiv)

//DCCregister
#define	F1 0
#define	F2 1
#define	F3 2
#define	F4 3
#define	FL 4
#define	RICHTUNG 5
#define	RICHTUNGINV 6
#define	FAHRSTUFEN 7

// Portdefinitionen
#define LICHTV PB3
#define LICHTR PB2
#define PINF1 PB4
#define PINF2 PB3
#define PINF3 PB2
#define PINF4 PB0

// 10ms Pause für Warten auf Inaktivität,
// prescaler ist ck/64 -> 255-(0,01/(1/1200000*64))
#define TVAL 67		// Wert für Timer0
//********************************
//* Registerdef.					*
//********************************

adresse	= 0	// eigene Adresse
vist = 1	// momentane Geschwindigkeit
vsoll = 2	// Soll Geschwindigkeit
incwert = 3	// Beschleunigungsvariable
decwert = 4	// Bremsvariable
inccnt	= 5	// Beschleunigungszhler/Bremszhler

flpwm	= 7	// Licht PWM in % 0=100
vstart	= 8	// Startspannung
progtest = 9	// Testbyte fr CV Schreiben
bytea = 10	// Empfangsbytespeicher
byteb = 11
bytec =	12
byted =	13
bytee =	14
bytef =	15
temp = 16	// temp register
null = 17	// register mit wert null
pwmaus = 18	// TCCR1 PWM Ausgang getrennt
pwman = 19	// TCCR1 PWM ein
fktsreg	= 20	// sreg sicherungsregister in FKTset Routine
inttemp = 21	// interrupt tempregister
intsreg = 22	// sreg sicherungsregister
flag = 23	// Flagregister
							//	Bit0 = 1 -> resetbit
							//	Bit1 = 1 -> CV29bit6=1 zur Progumschaltung Licht PWM fr RocoMaus
							//	Bit2 = 1 -> CV29bit5=1 zur Progumschaltung Motor PWM fr RocoMaus
							//	Bit3 = 1 -> Vsoll im EEProm merken
							//	Bit4     -> das neue bit
							//	Bit5 = 1 -> neues bit da	
							//	Bit6 = 1 -> bitzeit messen aktiv
							//	Bit7 = 1 -> lnger als 10000us keine bits eingetroffen
bitzaehler = 24	// Bitzaehler beim Empfang
eead = 25	// eepromaddresse
dccreg = 26	// DCC Register fr Funktionen und Fahrtrichtung
							//	Bit7 = 1 -> 28 Fahrstufen (0 = 14 Fahrstufen)
							//	Bit6 = 1 -> Fahrtrichtung reversiert
							//	Bit5 Fahrtrichtung 1=vorwrts, 0=rckwrts
							//   Bit4 FL Licht ein/aus	
							//	Bit3 F4 	
							//	Bit2 F3
							//	Bit1 F2
							//	Bit0 F1
vmax =	27	// Begrenzung v_max auf 50/62,5%/75/88,5%/100%
pwmzaehler = 28	// Zaehler fr Licht PWM
eevist	= 29	// EE Adresse in der Vist im EEPROM gespeichert wird
grundcnt = 30	// Zhler fr Grundverzgerung

// global symbols, IRQ vectors
.global main
.global SIG_INTERRUPT0
.global SIG_OVERFLOW0

.section .eeprom,"aw"
//CV Register Grundeinstellungen im eeprom
.byte 0xff
.byte 0x1			//CVregister 1 = Dekoderaddresse
.byte 0x1			//CVregister 2 = Startspannung 0-255 0 funktion aus 255 immer V_max
.byte 0x0			//CVregister 3 = Beschleunigungsrate ca 0-255 zeitstufen
.byte 0x0			//CVregister 4 = Verzgerungsrate ca 0-255 zeitstufen
.byte 0x4			//CVregister 5 = V_max Begrenzung auf 4=50% 3=62,5% 2=75% 1=88,5% 0=100%
.byte 70			//CVregister 6 = Licht PWM dunkel Zeit in %	(0 immer an 99 = 99% aus)	
.byte 32			//CVregister 7 = Zeiger auf Vist Speicherplatz Standart ist 32
.byte 0xff
.byte 5			//CVregister 9 = Motor PWM Periode (0=6452Hz 1=3226Hz bis 7=50Hz) 5=202Hz
.byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
.byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
.byte 0b00010000	//CVregister 29 Konfigurationsregister (zum Licht PWM programmieren+64=82 Motor PWM+32=50)
				//Funktionen:	bit0=1 Richtungstausch
				//(18)			bit1=0 14/ =1 28 Fahrstufen
				//			bit4=1 Vsoll in EEprom merken
				//			bit5=1 nächster ProgBefehl Motor PWM
				//			bit6=1 nächster ProgBefehl Licht PWM
.byte 0xff, 0xff
.byte 0x0, 0x0			//bit1-7 Vist bit0 Richtung wenn in CV 29 gefordert

#endif
